var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Connector } from '@web3-react/types';
export class WalletConnect extends Connector {
    /**
     * @param options - Options to pass to `@walletconnect/ethereum-provider`
     * @param connectEagerly - A flag indicating whether connection should be initiated when the class is constructed.
     */
    constructor(actions, options, connectEagerly = true) {
        super(actions);
        this.disconnectListener = (error) => {
            this.actions.reportError(error);
        };
        this.chainChangedListener = (chainId) => {
            this.actions.update({ chainId });
        };
        this.accountsChangedListener = (accounts) => {
            this.actions.update({ accounts });
        };
        this.options = options;
        if (connectEagerly) {
            this.eagerConnection = this.initialize(true);
        }
    }
    initialize(connectEagerly) {
        return __awaiter(this, void 0, void 0, function* () {
            let cancelActivation;
            if (connectEagerly) {
                cancelActivation = this.actions.startActivation();
            }
            return import('@walletconnect/ethereum-provider').then((m) => {
                this.provider = new m.default(this.options);
                this.provider.on('disconnect', this.disconnectListener);
                this.provider.on('chainChanged', this.chainChangedListener);
                this.provider.on('accountsChanged', this.accountsChangedListener);
                if (connectEagerly) {
                    if (this.provider.connected) {
                        return Promise.all([
                            this.provider.request({ method: 'eth_chainId' }),
                            this.provider.request({ method: 'eth_accounts' }),
                        ])
                            .then(([chainId, accounts]) => {
                            if (accounts === null || accounts === void 0 ? void 0 : accounts.length) {
                                this.actions.update({ chainId, accounts });
                            }
                            else {
                                throw new Error('No accounts returned');
                            }
                        })
                            .catch((error) => {
                            console.debug('Could not connect eagerly', error);
                            cancelActivation();
                        });
                    }
                    else {
                        cancelActivation();
                    }
                }
            });
        });
    }
    /** {@inheritdoc Connector.activate} */
    activate() {
        return __awaiter(this, void 0, void 0, function* () {
            this.actions.startActivation();
            if (!this.eagerConnection) {
                this.eagerConnection = this.initialize(false);
            }
            yield this.eagerConnection;
            try {
                // these are sequential instead of parallel because otherwise, chainId defaults to 1 even
                // if the connecting wallet isn't on mainnet
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                const accounts = yield this.provider.request({ method: 'eth_requestAccounts' });
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                const chainId = yield this.provider.request({ method: 'eth_chainId' });
                this.actions.update({ chainId, accounts });
            }
            catch (error) {
                // this condition is a bit of a hack :/
                // if a user triggers the walletconnect modal, closes it, and then tries to connect again, the modal will not trigger.
                // the logic below prevents this from happening
                if (error.message === 'User closed modal') {
                    yield this.deactivate();
                }
                this.actions.reportError(error);
            }
        });
    }
    /** {@inheritdoc Connector.deactivate} */
    deactivate() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.provider) {
                yield this.provider.disconnect();
                this.provider.off('disconnect', this.disconnectListener);
                this.provider.off('chainChanged', this.chainChangedListener);
                this.provider.off('accountsChanged', this.accountsChangedListener);
                this.provider = undefined;
                this.eagerConnection = undefined;
            }
        });
    }
}
